/*
 * Copyright (c) 2002-2005, In-VIGO Team
 * Advanced Computing and Information Systems (ACIS) Laboratory
 * University of Florida
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * sk.h		This module implements inter-proxy session key authentication.
 *
 * Author:	Ming Zhao
 */

#include <stdio.h>
#include <string.h>
#include "logging.h"
#include <rpc/rpc.h>
#include <unistd.h>
#include "nfsd.h"
#include "sk.h"

int		client_id = 1;
int		send_sk_enabled = 0;
int		match_sk_enabled = 0;
char		send_sk[16];
char		recv_sk[16];
char		match_sk[16*MAX_CLIENT_NUM];
char		*send_sk_path = NULL;
char  		*match_sk_path = NULL;
int		sk_no = 0; /* number of session keys */

extern int refresh_authentication_handle_uid;
extern int refresh_authentication_handle_gid;
extern int nfsportin;

int sk_init(char *send_sk_path, char *match_sk_path)
{
	FILE *fp;
	int reads, index=0;

	if(send_sk_enabled) {
		if((fp = fopen(send_sk_path, "rb")) == NULL) {
			Dprintf(L_ERROR, "Error: Can't open %s. Session key authentication disabled.\n", send_sk_path);
			send_sk_enabled = 0;
			return -1;
		} else {
			if((reads = fread(&send_sk, 1, 16, fp)) != 16) {
				Dprintf(L_ERROR, "Error: Key is shorter than 128bits. Session key authentication disabled.\n");
				send_sk_enabled = 0;
				return -1;
			}
			Dprintf(D_AUTH, "Send session key enabled\n");
			hexdump(send_sk, 16, D_AUTH);
			fclose(fp);
		}
	}

	if(match_sk_enabled) {
		sk_no = 0;
		if((fp = fopen(match_sk_path, "rb")) == NULL) {
			Dprintf(L_ERROR, "Error: Can't open %s. Session key authentication disabled\n", match_sk_path);
			match_sk_enabled = 0;
			return -1;
		} else {
			Dprintf(D_AUTH, "proxy server authorizes the following session keys:\n");
			while(1) {
				if(sk_no == MAX_CLIENT_NUM) {
					Dprintf(L_ERROR, "Error: Exeed maximum allowed client number. Session key authentication disabled\n");
					match_sk_enabled = 0;
					return -1;
				}
				if((reads = fread(&match_sk[index], 1, 16, fp)) == 0) {
					bzero(&match_sk[index], 16);
					break;
				}
				if(reads != 16) {
					Dprintf(L_ERROR, "Error: Key is shorter than 128bits. Session key authentication disabled.\n");
					match_sk_enabled = 0;
					return -1;
				}
				hexdump(&match_sk[index], 16, D_AUTH);
				index += 16; sk_no ++;
			}
			fclose(fp);
		}
	}
	return 0;
}

int sk_pack(char *machname)
{
	char temp[1024];

	if(strlen(machname) > 1024-16-1) {
		Dprintf(L_ERROR, "Machine name is too long to piggyback session key\n");
		return(-1);
	}
	if(memcpy(temp, send_sk, 16) == NULL) {
		Dprintf(L_ERROR, "Memory copy error\n");
		return(-1);
	}

	if(strcpy(temp+16, machname) == NULL) {
		Dprintf(L_ERROR, "String copy failed\n");
		return(-1);
	}

	if(strcpy(machname, temp) == NULL) {
		Dprintf(L_ERROR, "String copy failed\n");
		return(-1);
	}
	Dprintf(D_AUTH, "sk_pack:\n");
	hexdump(machname, strlen(machname), D_AUTH);

	return 0;
}

/* session key authentication */
int auth_sk(struct svc_req *rqstp)
{
	struct authunix_parms *unix_cred;
	char machname[1024];
	int i = 0;

	if (rqstp->rq_cred.oa_flavor == AUTH_UNIX) {
		unix_cred = (struct authunix_parms *) rqstp->rq_clntcred;

		if(memcpy(recv_sk, unix_cred->aup_machname, 16) == NULL) {
			Dprintf(L_ERROR, "Memory copy failed\n");
			return 0;
		}
		Dprintf(D_AUTH, "auth_sk:\n");
		hexdump(recv_sk, 16, D_AUTH);

		strcpy(machname, unix_cred->aup_machname+16);

		strcpy(unix_cred->aup_machname, machname);
		while(i < sk_no) {
			if(memcmp(&match_sk[i*16], recv_sk, 16) == 0) {
				Dprintf(D_AUTH, "Session key authorization successful (%d)\n", i+1);
				return i+1;
			}
			i ++;
		}
	}

	return 0;
}
